@@ -6,7 +6,6 @@ package URPM;
 use strict;
 use warnings;
 use Config;
-use Math::Int64 ':native_if_available', 'int64';
 
 # perl_checker: require URPM
 
@@ -337,6 +336,7 @@ sub _find_required_package__sort {
 	my ($best, @other) = sort {
 	      $a->[1] <=> $b->[1] #- we want the lowest (ie preferred arch)
 	      || $b->[2] <=> $a->[2] #- and the higher score
+	      || $b->[0]->compare_pkg($a->[0]) #- then by EVR (for upgrade)
 	      || $a->[0]->fullname cmp $b->[0]->fullname; #- then by name
 	} map {
 	    my $score = 0;
@@ -360,7 +360,7 @@ sub _find_required_package__sort {
     if ($urpm->{media}) {
 	@chosen_with_score = sort {
 	    $a->[2] != $b->[2] ? 
-	       $a->[0]->id <=> $b->[0]->id : 
+	       $b->[0]->compare_pkg($a->[0]) :
 	       $b->[1] <=> $a->[1] || $b->[0]->compare_pkg($a->[0]);
 	} map { [ $_, _score_for_locales($urpm, $db, $_), pkg2media($urpm->{media}, $_) ] } @chosen;
     } else {
@@ -641,11 +641,12 @@ sub unsatisfied_requires {
 	    #- check on the selected package if a provide is satisfying the resolution (need to do the ops).
 	    foreach (grep { exists $state->{selected}{$_} } keys %{$urpm->{provides}{$n} || {}}) {
 		my $p = $urpm->{depslist}[$_];
+		next if $p->arch eq 'src'; # ignore provides from SRPM (new in rpm-4.16)
 		!$urpm->{provides}{$n}{$_} || $p->provides_overlap($prop) and next REQUIRES;
 	    }
 
 	    #- check if the package itself provides what is necessary.
-	    $pkg->provides_overlap($prop) and next REQUIRES;
+	    $pkg->arch ne 'src' and $pkg->provides_overlap($prop) and next REQUIRES;
 
 	    #- check on installed system if a package which is not obsoleted is satisfying the require.
 	    my $satisfied = 0;
@@ -1210,7 +1211,7 @@ sub resolve_requested__no_recommends_ {
 	    } else {
 		_set_flag_installed_and_upgrade_if_no_newer($db, $pkg);
 
-		if ($pkg->flag_installed && !$pkg->flag_upgrade && !$urpm->{options}{downgrade}) {
+		if ($pkg->flag_installed && !$pkg->flag_upgrade && !$urpm->{options}{downgrade} && !$urpm->{options}{reinstall}) {
 		    _no_more_recent_installed_and_providing($urpm, $db, $state, $pkg, $dep->{required}) or next;
 		}
 	    }
@@ -1284,7 +1285,7 @@ sub resolve_requested__no_recommends_ {
 
 	# safety:
 	if ($count++ > 50000) {
-	    die("detecting looping forever while trying to resolve dependancies.\n"
+	    die("detecting looping forever while trying to resolve dependencies.\n"
 		. "Aborting... Try again with '-vv --debug' options");
 	}
     } while (@diff_provides || @properties || @choices);
@@ -1417,7 +1418,7 @@ sub _unselect_package_deprecated_by_property {
 		#- remove this package from the list of packages to install,
 		#- unless urpmi was invoked with --allow-force 
 		#- (in which case rpm could be invoked with --oldpackage)
-		if (!$urpm->{options}{'allow-force'} && !$urpm->{options}{downgrade}) {
+		if (!$urpm->{options}{'allow-force'} && !$urpm->{options}{downgrade} && !$urpm->{options}{reinstall}) {
 		    #- since the originally requested packages (or other
 		    #- non-installed ones) could be unselected by the following
 		    #- operation, remember them, to warn the user
@@ -1429,6 +1430,10 @@ sub _unselect_package_deprecated_by_property {
 		}
 	    } elsif ($satisfied) {
 		$obsoleted = 1;
+	    } elsif ($urpm->{options}{reinstall}) {
+		# So that we do not ask for "The following package has to be removed for others to be upgraded:
+		# foo-V-R (in order to install foo-V-R) (y/N)"
+		return;
 	    }
 	} elsif ($satisfied) {
 	    $obsoleted = 1;
@@ -1502,6 +1507,8 @@ sub _handle_diff_provides {
 		    $_->obsoletes_overlap($p->name . " == " . $p->epoch . ":" . $p->version . "-" . $p->release))
 		   && (!strict_arch($urpm) || strict_arch_check($p, $_));
 	     } @packages;
+	#- don't promote an obsolete package (mga#23223)
+	@packages = grep { _find_packages_obsoleting($urpm, $state, $_) == 0 } @packages;
 
 	if (!@packages) {
 	    @packages = _find_packages_obsoleting($urpm, $state, $p);
@@ -1559,6 +1566,8 @@ sub _handle_conflict {
       $p->version . "-" . $p->release;
     my @packages = grep { $_->name eq $p->name } find_candidate_packages($urpm, $need_deps, $state->{rejected});
     @packages = grep { ! $_->provides_overlap($property) } @packages;
+    #- don't promote an obsolete package (mga#23223)
+    @packages = grep { _find_packages_obsoleting($urpm, $state, $_) == 0 } @packages;
 
     if (!@packages) {
 	@packages = _find_packages_obsoleting($urpm, $state, $p);
@@ -1838,7 +1847,6 @@ sub _selected_size_filesize {
     my ($urpm, $state, $compute_filesize) = @_;
     my ($size, $filesize, $bad_filesize);
 
-    $size = int64(0); # make sure to use Int64, even on 32bit
     foreach (keys %{$state->{selected} || {}}) {
 	my $pkg = $urpm->{depslist}[$_];
 	$size += $pkg->size;
@@ -2033,10 +2041,10 @@ sub request_packages_to_upgrade {
 	}
 	if (my @pkgs = _choose_bests_obsolete($urpm, $db, $pkg_installed, _find_packages_obsoleting($urpm, $state, $pkg_installed))) {
 	    if (@pkgs == 1) {
-		$pkg and $urpm->{debug_URPM}("auto-select: prefering " . $pkgs[0]->fullname . " obsoleting " .  $pkg_installed->fullname . " over " . $pkg->fullname) if $urpm->{debug_URPM};
+		$pkg and $urpm->{debug_URPM}("auto-select: preferring " . $pkgs[0]->fullname . " obsoleting " .  $pkg_installed->fullname . " over " . $pkg->fullname) if $urpm->{debug_URPM};
 		$pkg = $pkgs[0];
 	    } elsif (@pkgs > 1) {
-		$urpm->{debug_URPM}("auto-select: multiple packages (" . join(' ', map { scalar $_->fullname } @pkgs) . ") obsoleting " . $pkg_installed->fullname) if $urpm->{debug_URPM};
+		$urpm->{debug_URPM}("auto-select: multiple packages (" . join(' ', sort(map { scalar $_->fullname } @pkgs)) . ") obsoleting " . $pkg_installed->fullname) if $urpm->{debug_URPM};
 		$pkg = undef;
 	    }
 	}
